unit f_main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ExtCtrls, StdCtrls, rImprovedComps, Grids, ValEdit,
  PropSaveMain, ComObj, rStringGridEd, rHTMLLabel, ComCtrls, AppEvnts;

type
  TMain = class(TForm)
    gbTop: TrGroupBox;
    ImageList1: TImageList;
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    mErr: TrStringsHolder;
    gbBottom: TrGroupBox;
    btAbort: TButton;
    btClose: TButton;
    Timer: TTimer;
    pc: TrPageControlEx;
    ts1: TTabSheet;
    ts2: TTabSheet;
    btAddFile: TButton;
    rHTMLLabel1: TrHTMLLabel;
    gbRun: TrGroupBox;
    rgOutFrm: TRadioGroup;
    btConvert: TButton;
    chDel: TCheckBox;
    rGroupBox2: TrGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    cbMask: TComboBox;
    chSubDir: TCheckBox;
    btLoad: TButton;
    gbClient: TrGroupBox;
    gr: TrStringGridEd;
    btClearFiles: TButton;
    OpenDlg: TOpenDialog;
    AppEvents: TApplicationEvents;
    cbDir: TComboBox;
    btDir: TButton;
    btClearHist: TrHTMLLabel;
    chIgnoreError: TCheckBox;
    lblHelp: TrURLLabel;
    lblAuthor: TrHTMLLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btConvertClick(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure eDirChange(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btAbortClick(Sender: TObject);
    procedure rgInFrmClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btAddFileClick(Sender: TObject);
    procedure gbTopDropFiles(Sender: TObject; X, Y: Integer);
    procedure btClearFilesClick(Sender: TObject);
    procedure AppEventsIdle(Sender: TObject; var Done: Boolean);
    procedure grCellClick(Sender: TObject; ACol, ARow: Integer);
    procedure chDelClick(Sender: TObject);
    procedure btDirClick(Sender: TObject);
    procedure btClearHistAnchorClick(Sender: TObject; LinkName: string);
    procedure lblHelpClick(Sender: TObject);
  private
    c0Title, c2Title: string;
    WordApplication, WordDocument: Variant;
    FCount, FSize, FSize2: Integer;
    maindir, inmask: string;
    AutoRun, AutoLoad, Aborted, IsRunning: Boolean;
    procedure RefState;
    function FormatSize(Size: Int64): string;
    procedure ClearList;
    procedure ConvertDoc(idx: Integer);
    procedure LoadFiles;
    procedure AddFile(fn: string);
    function GetFullFileName(ARow: Integer): string;
  public
  end;

var
  Main: TMain;

implementation

{$R *.dfm}
uses res, rstring, rtool, rfiletool, rdlg, rDialogsEx;

procedure TMain.FormCreate(Sender: TObject);
var
  i: Integer;
  s: string;
begin
  IsRunning:=false;
  AutoRun:=false;
  AutoLoad:=false;
  ClearList;

  cbDir.Text:=GetAppDir;
  c0Title:=gr.Columns[0].TitleCaption;
  c2Title:=gr.Columns[2].TitleCaption;
  OpenDlg.Filter:=rstring.sFileTypeRTF + '|' + sFileTypeDOC + '|' + sFileTypeAll;

  for i:=1 to ParamCount do
  begin
    self.PSFile.DisableAll:=true;
    s:=ParamStr(i);
    if Pos('dir=', s) = 1 then
    begin
      cbDir.Text:=Trim(copy(s, 5, length(s)));
      AutoLoad:=true;
    end;
    if Pos('mask=', s) = 1 then
      cbMask.Text:=Trim(copy(s, 6, length(s)));
    if s = 'subdir' then
      self.chSubDir.Checked:=true;
    if Pos('file=', s) = 1 then
      AddFile(Trim(copy(s, 6, length(s))));

    if s = 'out-rtf' then
      self.rgOutFrm.ItemIndex:=0;
    if s = 'out-doc' then
      self.rgOutFrm.ItemIndex:=1;
    if s = 'out-pdf' then
      self.rgOutFrm.ItemIndex:=2;
    if s = 'del' then
      self.chDel.Checked:=true;

    if s = 'autorun' then
      AutoRun:=true;
  end;
  if AutoLoad then
    LoadFiles;
  if AutoRun then
    Timer.Enabled:=true;
end;

procedure TMain.FormResize(Sender: TObject);
begin
  gr.ColWidths[0]:=gr.Width - 340;
  gr.ColWidths[1]:=50;
  gr.ColWidths[2]:=120;
  gr.ColWidths[3]:=50;
  gr.ColWidths[4]:=40;
  gr.ColWidths[5]:=50;
end;

procedure TMain.TimerTimer(Sender: TObject);
begin
  Timer.Enabled:=false;
  if btConvert.Enabled then
    btConvert.Click;
  Close;
end;

procedure TMain.RefState;
begin
  btConvert.Enabled:=not IsRunning and (FCount > 0);

  gbTop.Enabled:=not IsRunning;
  btClose.Enabled:=not IsRunning;
  btAbort.Visible:=IsRunning;
  btAbort.Enabled:=not Aborted;

  gr.Columns[0].TitleCaption:=c0Title + ' ' + Format(cNumOfFiles, [FCount, FormatSize(FSize)]);
  gr.Columns[2].TitleCaption:=c2Title + ' ' + Format('(%s)', [FormatSize(FSize2)]);
  Application.ProcessMessages;
end;

procedure TMain.AppEventsIdle(Sender: TObject; var Done: Boolean);
begin
  RefState;
end;

function TMain.FormatSize(Size: Int64): string;
begin
  Size:=Size div 1024;
  if Size < 1000 then
    Result:=Format('%d kB', [Size])
  else
    if Size < 10000 then
      Result:=Format('%0.1f MB', [Size / 1024])
    else
      Result:=Format('%d MB', [Size div 1024]);
end;

procedure TMain.btLoadClick(Sender: TObject);
begin
  LoadFiles;
end;

procedure TMain.chDelClick(Sender: TObject);
begin
  if false and self.Visible and chDel.Checked then
    if DlgQ2(sDeleteOrigQ, sContinueQ) <> mrYes then
      chDel.Checked:=false;
end;

procedure TMain.btAddFileClick(Sender: TObject);
var
  i: Integer;
begin
  if OpenDlg.Execute then
  begin
    for i:=0 to OpenDlg.Files.Count - 1 do
      AddFile(OpenDlg.Files[i]);
  end;
end;

procedure TMain.gbTopDropFiles(Sender: TObject; X, Y: Integer);
  procedure ProcDD(O: TrGroupBox);
  var
    i: Integer;
  begin
    for i:=0 to O.DropFilesCount - 1 do
      AddFile(O.DropFileNames[i]);
  end;

begin
  ProcDD(Sender as TrGroupBox);
end;

procedure TMain.grCellClick(Sender: TObject; ACol, ARow: Integer);
var
  r: Integer;
begin
  if (FCount > 0) then
  begin
    if ACol = 4 then
    begin
      RunFileSimple(GetFullFileName(ARow));
    end;
    if ACol = 5 then
    begin
      FSize:=FSize - rfiletool.GetFileSize(GetFullFileName(ARow));
      for r:=ARow to gr.RowCount - 2 do
        gr.Rows[r]:=gr.Rows[r + 1];
      // for c:=0 to 3 do  gr.Cells[c, r]:=gr.Cells[c, r + 1];
      gr.Rows[gr.RowCount - 1].Clear;
      Dec(FCount);
      if gr.RowCount > 2 then
        gr.RowCount:=gr.RowCount - 1;
    end;
  end;
end;

procedure TMain.ClearList;
begin
  FCount:=0;
  FSize:=0;
  FSize2:=0;
  gr.RowCount:=2;
  gr.Rows[1].Clear;
end;

procedure TMain.LoadFiles;

  procedure SearchDir(subdir: string);
  var
    F: TSearchRec;
  begin
    if FindFirst(maindir + subdir + inmask, faAnyFile, F) = 0 then
      repeat
        Inc(FCount);
        gr.RowCount:=FCount + 1;
        gr.Cells[0, FCount]:='..' + subdir + F.Name;
        gr.Cells[1, FCount]:=FormatSize(F.Size);
        FSize:=FSize + F.Size;
        gr.Cells[2, FCount]:='';
        gr.Cells[3, FCount]:='';
        gr.Cells[4, FCount]:=sOpen;
        gr.Cells[5, FCount]:=sRemove;

        Application.ProcessMessages;
        if Aborted then
          Exit;
        if (FCount > 100) and (FCount mod 100 = 0) then
          RefState;
      until FindNext(F) <> 0;
      FindClose(F);

    if not chSubDir.Checked then
      Exit;

    if FCount > 100 then
    begin
      IsRunning:=true;
      btAbort.Caption:=sCancelAction;
      // RefState;
    end;

    if FindFirst(maindir + subdir + '*.*', faDirectory, F) = 0 then
      repeat
        if F.Name = '.' then
          Continue;
        if F.Name = '..' then
          Continue;
        if (F.Attr and faDirectory) <> 0 then
        begin
          SearchDir(subdir + F.Name + '\');
        end;
      until FindNext(F) <> 0;
      FindClose(F);
  end;

begin
  ClearList;
  maindir:=SetEndOfPath(Trim(cbDir.Text));
  if cbDir.Items.IndexOf(maindir) < 0 then
    cbDir.Items.Add(maindir);
  inmask:=cbMask.Text;
  if cbMask.Items.IndexOf(inmask) < 0 then
    cbMask.Items.Add(inmask);
  WCursor;
  Aborted:=false;
  try
    SearchDir('');
  finally
    IsRunning:=false;
    NCursor;
  end;
end;

procedure TMain.AddFile(fn: string);
var
  fs: Int64;
begin
  Inc(FCount);
  gr.RowCount:=FCount + 1;
  gr.Cells[0, FCount]:=fn;
  fs:=rfiletool.GetFileSize(fn);
  gr.Cells[1, FCount]:=FormatSize(fs);
  FSize:=FSize + fs;
  gr.Cells[2, FCount]:='';
  gr.Cells[3, FCount]:='';
  gr.Cells[4, FCount]:=sOpen;
  gr.Cells[5, FCount]:=sRemove;
end;

function TMain.GetFullFileName(ARow: Integer): string;
var
  fn: string;
begin
  fn:=gr.Cells[0, ARow];
  if copy(fn, 1, 2) = '..' then
    fn:=maindir + copy(fn, 3, length(fn));
  Result:=fn;
end;

procedure TMain.rgInFrmClick(Sender: TObject);
begin
  LoadFiles;
end;

procedure TMain.btAbortClick(Sender: TObject);
begin
  Aborted:=true;
  RefState;
end;

procedure TMain.btClearFilesClick(Sender: TObject);
begin
  ClearList;
end;

procedure TMain.btClearHistAnchorClick(Sender: TObject; LinkName: string);
begin
  cbDir.Items.Clear;
  cbMask.Items.Clear;
end;

procedure TMain.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TMain.btConvertClick(Sender: TObject);
var
  i: Integer;
  log: string;
begin
  { if rgInFrm.ItemIndex = rgOutFrm.ItemIndex then
    begin
    DlgW(sInOutSameW);
    Exit;
    end;
    }
  gr.Cols[2].Clear;
  gr.Cols[3].Clear;
  FSize2:=0;
  Aborted:=false;
  IsRunning:=true;
  mErr.Strings.Clear;
  btAbort.Caption:=sMSWordOpening;
  RefState;
  try
    try
      WordApplication:=CreateOleObject('Word.Application');
      try
        // WordApplication.visible:=true;
        gr.TopRow:=1;
        if Aborted then
          Exit;
        for i:=1 to FCount do
        begin
          ConvertDoc(i);
          if i > 6 then
            gr.TopRow:=i - 6;
          btAbort.Caption:=Format(sNumOfNumFinished, [i, FCount]) + ', ' + sCancelAction;
          RefState;
          if Aborted then
            Exit;
        end;
      finally
        WordApplication.Quit(false)
      end;
    except
      mErr.Strings.Add(sMSWordNotFound);
      DlgE(sMSWordNotFound);
    end;
  finally
    // btConvert.Enabled:=false;
    IsRunning:=false;
    log:='';
    // if AutoRun then log:=FormatDateTime('_yyyymmdd_hhnnss', Now);
    mErr.Strings.SaveToFile(GetAppDir + 'log' + log + '.txt');
  end;
end;

procedure TMain.ConvertDoc(idx: Integer);
const
  wdFormatDocument = $00000000;
  wdFormatTemplate = $00000001;
  wdFormatText = $00000002;
  wdFormatTextLineBreaks = $00000003;
  wdFormatDOSText = $00000004;
  wdFormatDOSTextLineBreaks = $00000005;
  wdFormatRTF = $00000006;
  wdFormatUnicodeText = $00000007;
  wdFormatEncodedText = $00000007;
  wdFormatHTML = $00000008;
  wdExportFormatPDF = 17;
var
  fin, fout, ext: string;
  frm: OleVariant;
  fs: Int64;
begin
  case rgOutFrm.ItemIndex of
    0:
      frm:=wdFormatRTF;
    1:
      frm:=wdFormatDocument;
    2:
      frm:=wdExportFormatPDF;
  end;
  case rgOutFrm.ItemIndex of
    0:
      ext:='.rtf';
    1:
      ext:='.doc';
    2:
      ext:='.pdf';
  end;
  fin:=GetFullFileName(idx);
  fout:=ChangeFileExt(fin, ext);
  if SameText(fin, fout) then
  begin
    gr.Cells[2, idx]:=sSkipSame;
    Exit;
  end;

  try
    WordDocument:=WordApplication.Documents.Open(fin, false, true, false);
    if rgOutFrm.ItemIndex = 2 then
      WordDocument.ExportAsFixedFormat(fout, frm)
    else
      WordDocument.SaveAs(fout, frm, EmptyParam, EmptyParam, false);
    WordDocument.Close(false);
    gr.Cells[2, idx]:=sConversionDone;
    fs:=rfiletool.GetFileSize(fout);
    gr.Cells[3, idx]:=FormatSize(fs);
    FSize2:=FSize2 + fs;
    mErr.Strings.Add(Format('%d: %s - %s', [idx, 'Ok', gr.Cells[0, idx]]));

    if self.chDel.Checked then
      try
        DeleteFile(fin);
      except
      end;
  except
    on E: Exception do
    begin
      gr.Cells[2, idx]:=sError + ' - ' + E.Message;
      mErr.Strings.Add(Format('%d: %s - %s (%s)', [idx, sError, gr.Cells[0, idx], E.Message]));
      if not AutoRun and not chIgnoreError.Checked then
        ShowErr(sMSWordError, E, false);
    end;
  end;

end;

procedure TMain.eDirChange(Sender: TObject);
begin
  ClearList;
end;

procedure TMain.btDirClick(Sender: TObject);
var
  s: string;
begin
  s:=BrowseForFolder('', cbDir.Text);
  if s <> '' then
  begin
    cbDir.Text:=s;
    LoadFiles;
  end;
end;

procedure TMain.lblHelpClick(Sender: TObject);
begin
  RunFileSimple(GetAppDir + 'help.pdf');
end;

end.
